from __future__ import annotations

from dataclasses import dataclass
from typing import Optional

from analogai.deepthink.infrastructure.gateways.llm.enums.llm_provider import LlmProvider
from analogai.deepthink.infrastructure.gateways.llm.gateway import LlmGateway


@dataclass(frozen=True)
class LlmConfig:
    provider: Optional[LlmProvider] = None
    model: Optional[str] = None
    gateway_instance: Optional[LlmGateway] = None
