from __future__ import annotations

from typing import Iterable, List


class InterpreterRegistry:
    def __init__(self, interpreters: Iterable) -> None:
        self._interpreters: List = list(interpreters)

    def register(self, interpreter) -> None:
        self._interpreters.append(interpreter)

    def all(self) -> List:
        return list(self._interpreters)
